/***************************************************************************
                          mylistview.h  -  description
                             -------------------
    begin                : Mon Jun 18 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MYLISTVIEW_H
#define MYLISTVIEW_H

#include <qwidget.h>
#include <qlayout.h>
#include <qsizepolicy.h>
#include <qlistbox.h>
#include <qlineedit.h>

class QStringList;
class QPushButton;
class QPoint;

/**implements an editable stringlist
  *@author Rdiger Knrig
  */

class MyListView : public QWidget  {
   Q_OBJECT
public: 
	/**
		* Default-Konstruktor
		* @param items Liste mit den Strings, mit denen die ListView initialisiert wird.
		* @param parent Parent-Widget, s. Konstruktor QWidget
		* @param name Widget-Name, s. Konstruktor QWidget
		*/
	MyListView(QStringList &items,QWidget *parent=0, const char *name=0);
	/**
		* Default-Destruktor
		*/
	~MyListView();
  /** 
returns a QStringList with the listbox items
@return QStringList with the listbox items
 */
  QStringList getItemList();
protected:
	/** main layout of the widget */
	QGridLayout *mainlayout;
	/** button layout */
	QVBoxLayout *buttonlayout;
	/** layout for the property edit area */
	QHBoxLayout *propertylayout;
	/** new-item-button */
	QPushButton *newitembutton;
	/** append-item-button */
	QPushButton *appenditembutton;
	/** delete-item-button */
	QPushButton *deleteitembutton;
	/** move-up-button */
	QPushButton *moveupbutton;
	/** move-down-button */
	QPushButton *movedownbutton;
	/** central listview */
	QListBox *listview;
	/** property value edit entry*/
	QLineEdit *entry;
protected slots:
	/** transfers content of highlighted item to text entry */
	void slotSelectItem( QListBoxItem *curritem) { if(curritem) entry->setText(curritem->text()); }
  /** appends an item at the end of the list */
	void slotAppendItem();
	/** inserts an element */
  void slotInsertItem();
  /** activated when return is pressed in entry */
  void slotAcceptItem();
	/** deletes an entry if an item is selected */
	void slotDeleteItem() { if(listview->currentItem() >= 0) listview->removeItem(listview->currentItem()); }
  /** moves an item from index form to index to */
  void slotMoveContent(int from, int to);
	/** moves Item up */
	void slotMoveUpItem() { int index = listview->currentItem(); if(index > 0) slotMoveContent(index,index-1); }
	/** moves item down */
	void slotMoveDownItem() { int index = listview->currentItem(); if(index >= 0 && listview->item(index)->next() != 0) slotMoveContent(index,index+1); }
};

#endif
