/***************************************************************************
                          mymainwidget.cpp  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapp.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qfile.h>
#include <qradiobutton.h>
#include <qspinbox.h>
#include <qcheckbox.h>
#include <qdatastream.h>
#include <qdir.h>

#include "A2psfrontenddialog.h"
#include "mymainwidget.h"
#include "mysetupdialog.h"

extern "C"
{
#include <stdlib.h>
#include <unistd.h>
}

MyMainWidget::MyMainWidget(QWidget *parent, const char *name ) : A2psFrontEndMainWidget(parent,name)
{

	path = QDir::homeDirPath() + "/" + QString(configfilename);
	
	getPreferences();
		
	initWidgets();
	
	// connect cancel button and qapp->quit...
	QObject::connect(CancelButton,SIGNAL(clicked()),qApp,SLOT(quit()));
}

MyMainWidget::~MyMainWidget()
{

}

/** generate shell command */
void MyMainWidget::Execute(bool preview)
{
	// input file is a must-be, so check its entry...
	QString ifilename(InputFileNameEntry->text());
	QFile ifile(ifilename);
	if(!ifile.exists())
	{
		if(ifilename.isEmpty())
			{
				QMessageBox::information(this,"a2psfrontend","Please specify an input file!\n");
			}
			else
			{	
				QMessageBox::information(this,"a2psfrontend",\
				"input file\n" + InputFileNameEntry->text() + "\ndoes not exist!");
			}
		InputFileNameEntry->setFocus();
	}
	else
	{
		//QMessageBox::information(this,"a2psfrontend",makeCommandStr(preview));
		if(system(makeCommandStr(preview))) QMessageBox::information(this,"a2psfrontend",\
		"An error occured while executing a2ps!");
	}
}


/** going back to initial values */
void MyMainWidget::Reset()
{
	QLineEdit *entries[10] = {InputFileNameEntry,OutputFilenameEntry,HeaderText,leftTitle,centralTitle,\
	rightTitle,leftFooter,centralFooter,rightFooter,lowerText};
	
	for(int i=0; i < 10; i++) entries[i]->clear();
	
	QCheckBox *checkers[3] = {truncateCheck,DrawBordersCheck,NoHeaderCheck};
	
	// be careful... some of them could be checked by default!
	for(int i=0; i < 3; i++) checkers[i]->setChecked(false);
	
	QSpinBox *numselectors[10] = {copycselector,firstpageselector,lastpageselector,StyleDefinitionSpin,\
	StyleDefinitionSpin,MarginSizeSelector,LineNumberDistanceSel,FontSizeSelector,RowSelectorSpinBox,\
	ColumnSelectorSpinBox};
	
	for(int i=0; i < 10; i++) numselectors[i]->setValue(numselectors[i]->minValue());
	
	// default destination is a printer
	DestinationPrinter->setChecked(true);
	
	QComboBox *combos[6] = {CodingSystem,PapersizeSelectionCombo,PaperOrientationCombo,SidesSelector,\
	PrinterSelectCombo,MajorDirectionSelector};
	
	initWidgets();
	
	for(int i=0; i < 6; i++) combos[i]->setCurrentItem(0);
}

/** input file selection */
void MyMainWidget::pickInputFile()
{
	QString temp = QFileDialog::getOpenFileName(InputFileNameEntry->text(),"all files (*)",this);
	if(!temp.isNull()) InputFileNameEntry->setText(temp);
}

/** output file selection */
void MyMainWidget::pickOutputFile()
{
	QString temp = QFileDialog::getSaveFileName(OutputFilenameEntry->text(),"all files (*)",this);
	if(!temp.isNull()) OutputFilenameEntry->setText(temp);	
}

void MyMainWidget::explainStyle(int style)
{
 if(style)
 	{
 		textlayout->setDisabled(true);
 		StyleExplanation->setText(numprint.sprintf("%i virtual pages per sheet",style));
 	}
 	else
 	{
 		textlayout->setDisabled(false);
 		StyleExplanation->setText("user defined style");
 	}
}

/** generates a QString with the a2ps calling command */
QString MyMainWidget::makeCommandStr(bool preview)
{
	QString command = this->command + " ";
	
	// append input file name
	command += (InputFileNameEntry->text() + " ");
	// append medium specification
	command += ("-M " + PapersizeSelectionCombo->currentText() + " ");
	// append paper orientation
	command += ((PaperOrientationCombo->currentItem()) ? "-R " : "-r ");
	// more than one copy to print ?
	if(copycselector->value() != 1)
		{
			command += numprint.sprintf("-n %i ", copycselector->value());
		}
	// truncate long lines ?
	if(truncateCheck->isChecked()) command += "-c ";
	// how many sides per page ?
	command += numprint.sprintf("-s %i ",SidesSelector->currentItem()+1);
	// which pages ?
	if(firstpageselector->value() || lastpageselector->value())
		command += numprint.sprintf("-a %i-%i ",firstpageselector->value(),lastpageselector->value());
	// number of virtual pages per side
	if(StyleDefinitionSpin->value())
		command += numprint.sprintf("-%i ",StyleDefinitionSpin->value());
	else
	{
		command += numprint.sprintf("--columns=%i ",ColumnSelectorSpinBox->value());
		command += numprint.sprintf("--rows=%i ",RowSelectorSpinBox->value());
		
		if(FontSizeSelector->value()) command += numprint.sprintf("-f %i ",FontSizeSelector->value());
		command += ("--major=" + MajorDirectionSelector->currentText() + " ");
		if(MarginSizeSelector->value()) command += numprint.sprintf("--margin=%i ",MarginSizeSelector->value());
		if(LineNumberDistanceSel->value()) command += numprint.sprintf("--line-numbers=%i ",LineNumberDistanceSel->value());
		if(DrawBordersCheck->isChecked()) command += "-j ";
	}
	// specify coding system, if not default
	if(CodingSystem->currentText() != "default") command += ("-X " + CodingSystem->currentText() + " ");
	
	// get header specifications, when given
	if(NoHeaderCheck->isChecked()) command += "-B ";
	else if(!HeaderText->text().isEmpty()) command += ("--header=" + HeaderText->text() + " ");
	if(!leftTitle->text().isEmpty()) command += ("--left-title=" + leftTitle->text() + " ");
	if(!centralTitle->text().isEmpty()) command += ("--center-title=" + centralTitle->text() + " ");
	if(!rightTitle->text().isEmpty()) command += ("--right-title=" + rightTitle->text() + " ");
	if(!leftFooter->text().isEmpty()) command += ("--left-footer=" + leftFooter->text() + " ");
	if(!centralFooter->text().isEmpty()) command += ("--footer=" + centralFooter->text() + " ");
	if(!rightFooter->text().isEmpty()) command += ("--right-footer=" + rightFooter->text() + " ");
	if(!lowerText->text().isEmpty()) command += ("--underlay=" + lowerText->text() + " ");
	
	// specify output, must be the last included option!
	if(preview)
		{
			QString tempfilename;
			tempfilename.sprintf("gv%i.tmp",getpid());
			command += ("-o " + tempfilename + "\n" + viewer + " " + tempfilename + "\nrm -f " + tempfilename);
		}
		else
		{
			if(DestinationPrinter->isChecked())
			{
				if(PrinterSelectCombo->currentText() != "default")
					{
						command += ( "-P " + PrinterSelectCombo->currentText() + " ");
					}
			}
			else
			{
				command += ("-o " + OutputFilenameEntry->text() + " ");
			}
		}
	return(command);
}
/**
	* tries to read a ".a2psfrontend" configuration file from users home dir and sets
  * some default values - yes, we too support trashing users homedir (besides, kde
	* supports a better way of storing program configurations)
	*/
void MyMainWidget::getPreferences()
{
	QFile input(path);
	
	// defining some default values for security reasons		
		
	// set viewer command
	viewer = "gv";
	
	// set a2ps command
	command = "a2ps ";
	
	// define some coding systems, first entry mmust be default!
	csystems << "default" << "iso-8859-1";
	
	// define some printers, first entry mmust be default!
	printers << "default";
	
	// try to read configuration from file...
	if(input.open(IO_ReadOnly))
		{
			QDataStream s(&input);
			s >> command >> viewer >> csystems >> printers;
		}
	input.close();
}
/** invokes setup dialog */
void MyMainWidget::Setup()
{
	MySetupDialog dlg(path,&command,&viewer,&csystems,&printers);
	
	dlg.exec();
	
	initWidgets();
}

/**
	* init _some_ Widgets to default settings (actually combos for printers and coding
	* systems)
	*/
void MyMainWidget::initWidgets()
{
	// init coding system combo
	CodingSystem->clear();
	CodingSystem->insertStringList(csystems);
	
	// init printer combo	
	PrinterSelectCombo->clear();
	PrinterSelectCombo->insertStringList(printers);
}
