/***************************************************************************
                          mymainwidget.h  -  description
                             -------------------
    begin                : Mon Jun 11 2001
    copyright            : (C) 2001 by Rdiger Knrig
    email                : ruediger@knoerig.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MYMAINWIDGET_H
#define MYMAINWIDGET_H

#include <qwidget.h>
#include <A2psfrontenddialog.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qlabel.h>
#include <vector>

class QMenuBar;
class QPopupMenu;

const char configfilename[] = ".a2psfrontend";

/**main Dialog for a2ps frontend
  *@author Rdiger Knrig
  */
class MyMainWidget : public A2psFrontEndMainWidget  {
   Q_OBJECT
public: 
	MyMainWidget(QWidget *parent=0, const char *name=0);
	~MyMainWidget();	
protected:
  /**
  	* generates a QString with the a2ps calling command
  	* @param preview If true, delegate output to a viewer (@see viewer)
  	*/
  QString makeCommandStr(bool preview = false);
  /** tries to read a ".a2psfrontend" configuration file from users home dir and sets 
		* some default values - yes, we too support trashing users homedir (besides, kde
    * supports a better way of storing program configurations)
    */
  void getPreferences();
  /**
  	* init _some_ Widgets to default settings (actually combos for printers and coding
		* systems)
		*/
  void initWidgets();
  /** contains a QString with the shell comman invoking the desired ps viewer */
  QString viewer;
  /** contains a QString with the shell command invoking a2ps */
  QString command;
  /** contains absolute path to config file */
  QString path;
  /** used to produce QStrings in sprintf style */
  QString numprint;
	/** contains all predefined coding systems (iso-8859-1 e.g.) */
	QStringList	csystems;
	/** contains a list of known printers */
	QStringList printers;
	/** menubar */
	QMenuBar *menubar;
	/** popup-menu for configuration & about - dialog */
	QPopupMenu *setupmenu;
protected slots: // Protected slots
  /** user accepted settings, execute a2ps */
  void Accept() { Execute(false); }
  /** generate a preview */
  void Preview() { Execute(true); }
  /** executes the command string */
  void Execute(bool preview);
 /** return to defaults */
  void Reset();
  /** should invoke input file selection dialog */
  void pickInputFile();
  /** should invoke output filename selection dialog */
  void pickOutputFile();
  /**
  	*	changes style explanation text when style
		* combo changes and disable some widgets if necessary
		*/
  void explainStyle(int style);
  /** invokes setup dialog */
  void Setup();

};

#endif
